#this file is part of manqod
#manqod is distributed under the CDDL licence
#the author of manqod is Dobai-Pataky Balint(dpblnt@gmail.com)

module Gtk
	# Thread-safety stuff.
	# Loosely based on booh, by Guillaume Cottenceau.

	PENDING_CALLS_MUTEX = Mutex.new
	PENDING_CALLS = []

	def self.new_thread(&proc)
		Thread.new{
			begin
				Gtk.thread_protect(&proc)
			rescue =>err
				print "#{err}\n#{err.join("\n")}"
			end
		}
	end

	def self.thread_protect(&proc)
		if Thread.current == Thread.main
			proc.call
		else
			$gtk_pending_calls.synchronize {$gtk_pending_calls << proc}
		end
	end

	def self.thread_flush
		closure = nil
		continue = true
		begin
			$gtk_pending_calls.synchronize {
				closure = $gtk_pending_calls.shift
				continue = $gtk_pending_calls.size > 0
			}
			if closure
				closure.call
			end
		end while continue
	end

	def self.init_thread_protect
		$gtk_pending_calls = []
		$gtk_pending_calls.extend(MonitorMixin)
		Gtk.timeout_add(100) {Gtk.thread_flush;true}
	end

	class PrintSettings
		 	include Marshal
		def serialize
			dump(to_a)
		end
		
		def deserialize(serialized)
			if serialized
				begin
					psettings=load(serialized)
				rescue =>err
					edebug("loading print settings: #{err}","printing","error")
				end
				if psettings
					psettings.each{|v|
						set(v[0].to_s,v[1])
					}
				end
			end
			self
		end
	end
	
	class TreeIter
		def next_with_children
			ret=nil
			if self.has_child?
				ret=self.first_child
			else
				if clone.next!
					self.next!
					ret=self
					else
					if par=self.parent
						if par.clone.next!
							par.next!
							ret=par
						end
					end
				end
			end
			ret
		end
	end

  def self.is_windows?
     RUBY_PLATFORM.downcase.include?("mswin") || RUBY_PLATFORM.downcase.include?("mingw")
  end

  def self.is_linux?
     RUBY_PLATFORM.downcase.include?("linux")
  end
  
  def self.show_thread_changes
  	Gtk.thread_protect{
			while Gtk.events_pending? do 
				Gtk.main_iteration_do(false)
			end
		} unless Gtk.is_windows?
  end
end

module Gdk
	module Keyval
		begin
			GDK_KEY_F5
		rescue NameError
			constants.each{|key| 
				const_set("GDK_KEY_"+key[4..-1],eval(key.to_s))
			}
		end
	end
end

