#this file is part of manqod
#manqod is distributed under the CDDL licence
#the author of manqod is Dobai-Pataky Balint(dpblnt@gmail.com)

module ManqodCommon
	include Eprint
	
	CRITICAL = "critical"
	ERROR = "error"
	WARNING = "warning"
	NORMAL = "normal"
	INFO = "info"
	DEBUG = "debug"
	

	def backtrace_to_debug(c)
		r="#{c}\n"
		c.backtrace.each{|i| r=r+"\t#{i}\n"}
		r
	end

	def measure(d="",color="default")
		c=case color
			when "red" then 31
			when "blue" then 34
			when "green" then 32
			when "yellow" then 33
			when "magenta" then 35
			when "cyan" then 36
			when "white" then 37
			when "default" then 39
		end
		before_=Time.now
		yield
		time_diff=Time.now.to_f-before_.to_f;edebug("\e[#{c}m#{self} #{d} measured: "+sprintf("%.2fs",time_diff)+"\e[0m","time","normal")
	end

def lzero(s,n)
	while s.to_s.length<n
		s="0#{s}"
	end
	s
end

def number_format(f,d)
	s=f.to_s
	if i=s.index(".")
		p1=s[0 .. i-1]
		p2=(s+"0000")[i .. i+d]
		else
		p1=s
		p2=".0000"
	end
	if d==0
		p1.gsub(/(\d)(?=(\d\d\d)+(?!\d))/, "\\1,")
		else
		p1.gsub(/(\d)(?=(\d\d\d)+(?!\d))/, "\\1,") + p2
	end
end

	def eeval(command,context=self)
  	ret=nil
  	begin
			edebug("Running: #{command}","eval","debug")
			if context and b=context.getBinding then
				ret=eval(command, b)
				else
				ret=eval(command)
			end
		rescue SyntaxError, NameError => err
	  	tell_exception("Syntax error: #{command}",backtrace_to_debug(err),"eval","error",true,false)
		rescue  => err
#	  edebug("String doesn't run: #{command}\n"+backtrace_to_debug(b),"eval","error")
	  	tell_exception("String doesn't run: #{command}",backtrace_to_debug(err),"eval","error",true,false)
  	end
  	ret
	end

	def getBinding
		binding
	end

	def run_events(gtkobjectid,event_name)
		edebug("Running events #{event_name} for #{gtkobjectid}","events","debug")
		EventCache.instance.gtkobject_events(gtkobjectid,event_name){|event_command| eeval(event_command,self)}
#		edebug("Running events #{event_name} for #{gtkobjectid} finished.","events","debug")
	end

	def myexec(filename,filedata)
		Myexec.instance.myexec(filename,filedata)
	end

	def nick
		Nick.instance.get_nick
	end

	def nick_id
		Nick.instance.nick_id
	end

	def set_nick(new_nick)
		Nick.instance.set_nick(new_nick)
	end

	def set_manqod_db_uri(uri)
			ManqodDB.instance.set_manqod_db_uri(uri)
	end

	def reconnect_manqod_db
			ManqodDB.instance.connect_to_server
	end

	def manqod_db
		begin
			ManqodDB.instance.manqod_db
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end
	def cache
		ManqodDB.instance.cache
	end
	def admin_cache
		ManqodDB.instance.acache
	end
	
#	def prep_sql(isql)
#		ManqodDB.instance.manqod_db.client.escape_string(isql)
#	end

	def client
		begin
			ManqodDB.instance.manqod_db.client
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def client_query(sql)
		begin
			einfo("Client query: #{sql}","sql")
			ManqodDB.instance.manqod_db.client.query(sql)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def client_fields(sql)
		begin
			ManqodDB.instance.manqod_db.client.fields(sql).each{|row| yield row}
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def client_qrow(sql)
		begin
			ManqodDB.instance.manqod_db.client.qrow(sql)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def client_rows(sql)
		begin
			ManqodDB.instance.manqod_db.client.rows(sql).each{|row| yield row}
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def admin
		begin
			ManqodDB.instance.manqod_db.admin
		rescue =>err
			retry if warn("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def query(sql)
		begin
			einfo("query: #{sql}","sql")
			ret=ManqodDB.instance.manqod_db.query(sql)
			tell_exception("Database Error","Duplicate entry","sql","error") if ret == -1
			ret
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def add_where(sql,where)
		begin
			ret=ManqodDB.instance.manqod_db.add_where(sql,where)
			tell_exception("Database Error","Duplicate entry","sql","error") if ret == -1
			ret
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def qrow(sql,with_table=false)
		begin
			ManqodDB.instance.manqod_db.qrow(sql,with_table)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def rows(sql,with_table=false)
		begin
			i=ManqodDB.instance.manqod_db.rows(sql,with_table).each{|row| yield row}
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def admin_rows(sql,with_table=false)
#		ecode("#{sql}")
		begin
			ManqodDB.instance.manqod_db.admin.rows(sql,with_table).each{|row| yield row}
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def admin_qrow(sql,with_table=false)
		begin
			ManqodDB.instance.manqod_db.admin.qrow(sql,with_table)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def guess_table(sql,field)
		begin
			ManqodDB.instance.manqod_db.guess_table(sql,field)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end
	def guess_base(sql)
		begin
			ManqodDB.instance.manqod_db.guess_base(sql)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end
	def escape_string(sql)
		begin
			ManqodDB.instance.manqod_db.escape_string(sql)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def image_of_id(image_id=nil,resize=nil)
		begin
			DrbImages.instance.image_of_id(image_id,resize)
		rescue =>err
			ewarn(backtrace_to_debug(err))
		end
	end
	def client_image_of_id(image_id=nil,resize=nil)
		begin
			DrbImages.instance.image_of_id(image_id,resize,true)
		rescue =>err
			ewarn(backtrace_to_debug(err))
		end
	end


	def changed_ids_of_base(base,ids)
		begin
			ManqodDB.instance.manqod_db.changed_ids_of_base(base,ids,nick)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end

	def sendmail(*args)
		begin
			ManqodDB.instance.manqod_db.sendmail(*args)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end
	def send_message(*args)
		begin
			ManqodDB.instance.manqod_db.send_message(*args)
		rescue =>err
			retry if tell_exception("RunTime error",backtrace_to_debug(err),"server","error",false,true,err.to_s)
		end
	end
end

